@@ -2,7 +2,7 @@
  * vim:ts=4:sw=4:expandtab
  *
  * i3bar - an xcb-based status- and ws-bar for i3
- * © 2010-2011 Axel Wagner and contributors (see also: LICENSE)
+ * © 2010-2012 Axel Wagner and contributors (see also: LICENSE)
  *
  * outputs.c: Maintaining the output-list
  *
@@ -266,7 +266,7 @@ yajl_callbacks outputs_callbacks = {
  * Initiate the output-list
  *
  */
-void init_outputs() {
+void init_outputs(void) {
     outputs = smalloc(sizeof(struct outputs_head));
     SLIST_INIT(outputs);
 }
