@@ -10,8 +10,8 @@
  * compile-time.
  *
  */
-#ifndef _ALL_H
-#define _ALL_H
+#ifndef I3_ALL_H
+#define I3_ALL_H
 
 #include <assert.h>
 #include <stdbool.h>
@@ -35,6 +35,9 @@
 #include <xcb/xcb_keysyms.h>
 #include <xcb/xcb_icccm.h>
 
+#include <yajl/yajl_gen.h>
+#include <yajl/yajl_version.h>
+
 /* Contains compatibility definitions for old libxcb versions */
 #ifdef XCB_COMPAT
 #include "xcb_compat.h"
@@ -51,6 +54,7 @@
 #include "i3.h"
 #include "x.h"
 #include "click.h"
+#include "key_press.h"
 #include "floating.h"
 #include "config.h"
 #include "handlers.h"
@@ -75,6 +79,9 @@
 #include "scratchpad.h"
 #include "commands.h"
 #include "commands_parser.h"
+#include "config_directives.h"
+#include "config_parser.h"
 #include "fake_outputs.h"
+#include "display_version.h"
 
 #endif
