@@ -8,8 +8,8 @@
  *          events.  This code is from xcb-util.
  *
  */
-#ifndef _DEBUG_H
-#define _DEBUG_H
+#ifndef I3_DEBUG_H
+#define I3_DEBUG_H
 
 int handle_event(void *ignored, xcb_connection_t *c, xcb_generic_event_t *e);
 
