@@ -21,6 +21,9 @@
 #if defined(ELOG)
 #undef ELOG
 #endif
+#if defined(DLOG)
+#undef DLOG
+#endif
 /** ##__VA_ARGS__ means: leave out __VA_ARGS__ completely if it is empty, that
    is, delete the preceding comma */
 #define LOG(fmt, ...) verboselog(fmt, ##__VA_ARGS__)
@@ -38,6 +41,24 @@ extern int shmlog_size;
  */
 void init_logging(void);
 
+/**
+ * Opens the logbuffer.
+ *
+ */
+void open_logbuffer(void);
+
+/**
+ * Closes the logbuffer.
+ *
+ */
+void close_logbuffer(void);
+
+/**
+ * Checks if debug logging is active.
+ *
+ */
+bool get_debug_logging(void);
+
 /**
  * Set debug logging.
  *
