@@ -14,6 +14,9 @@
 #include <stdint.h>
 #include <pthread.h>
 
+/* Default shmlog size if not set by user. */
+extern const int default_shmlog_size;
+
 /*
  * Header of the shmlog file. Used by i3/src/log.c and i3/i3-dump-log/main.c.
  *
