@@ -7,8 +7,8 @@
  * xcb.c: Helper functions for easier usage of XCB
  *
  */
-#ifndef _XCB_H
-#define _XCB_H
+#ifndef I3_XCB_H
+#define I3_XCB_H
 
 #include "data.h"
 #include "xcursor.h"
@@ -45,6 +45,15 @@
                           XCB_EVENT_MASK_SUBSTRUCTURE_NOTIFY |   /* …subwindows get notifies */ \
                           XCB_EVENT_MASK_ENTER_WINDOW)           /* …user moves cursor inside our window */
 
+#define ROOT_EVENT_MASK   (XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT | \
+                           XCB_EVENT_MASK_BUTTON_PRESS | \
+                           XCB_EVENT_MASK_STRUCTURE_NOTIFY |         /* when the user adds a screen (e.g. video \
+                                                                            projector), the root window gets a \
+                                                                            ConfigureNotify */ \
+                           XCB_EVENT_MASK_POINTER_MOTION | \
+                           XCB_EVENT_MASK_PROPERTY_CHANGE | \
+                           XCB_EVENT_MASK_ENTER_WINDOW)
+
 #define xmacro(atom) xcb_atom_t A_ ## atom;
 #include "atoms.xmacro"
 #undef xmacro
