@@ -1,3 +1,5 @@
+#undef I3__FILE__
+#define I3__FILE__ "assignments.c"
 /*
  * vim:ts=4:sw=4:expandtab
  *
@@ -49,7 +51,7 @@ void run_assignments(i3Window *window) {
             if (command_output->needs_tree_render)
                 needs_tree_render = true;
 
-            free(command_output->json_output);
+            yajl_gen_free(command_output->json_gen);
         }
 
         /* Store that we ran this assignment to not execute it again */
