@@ -65,11 +65,12 @@ static void insert_con_into(Con *con, Con *target, position_t position) {
 }
 
 /*
- * This function detaches 'con' from its parent and inserts it at the given
- * workspace.
+ * This function detaches 'con' from its parent and puts it in the given
+ * workspace. Position is determined by the direction of movement into the
+ * workspace container.
  *
  */
-static void attach_to_workspace(Con *con, Con *ws) {
+static void attach_to_workspace(Con *con, Con *ws, direction_t direction) {
     con_detach(con);
     con_fix_percent(con->parent);
 
@@ -77,8 +78,13 @@ static void attach_to_workspace(Con *con, Con *ws) {
 
     con->parent = ws;
 
-    TAILQ_INSERT_TAIL(&(ws->nodes_head), con, nodes);
-    TAILQ_INSERT_TAIL(&(ws->focus_head), con, focused);
+    if (direction == D_RIGHT || direction == D_DOWN) {
+        TAILQ_INSERT_HEAD(&(ws->nodes_head), con, nodes);
+        TAILQ_INSERT_HEAD(&(ws->focus_head), con, focused);
+    } else {
+        TAILQ_INSERT_TAIL(&(ws->nodes_head), con, nodes);
+        TAILQ_INSERT_TAIL(&(ws->focus_head), con, focused);
+    }
 
     /* Pretend the con was just opened with regards to size percent values.
      * Since the con is moved to a completely different con, the old value
@@ -87,6 +93,43 @@ static void attach_to_workspace(Con *con, Con *ws) {
     con_fix_percent(ws);
 }
 
+/*
+ * Moves the given container to the closest output in the given direction if
+ * such an output exists.
+ *
+ */
+static void move_to_output_directed(Con *con, direction_t direction) {
+    Con *old_ws = con_get_workspace(con);
+    Con *current_output_con = con_get_output(con);
+    Output *current_output = get_output_by_name(current_output_con->name);
+    Output *output = get_output_next(direction, current_output, CLOSEST_OUTPUT);
+
+    if (!output) {
+        DLOG("No output in this direction found. Not moving.\n");
+        return;
+    }
+
+    Con *ws = NULL;
+    GREP_FIRST(ws, output_get_content(output->con), workspace_is_visible(child));
+
+    if (!ws) {
+        DLOG("No workspace on output in this direction found. Not moving.\n");
+        return;
+    }
+
+    attach_to_workspace(con, ws, direction);
+
+    /* fix the focus stack */
+    con_focus(con);
+
+    /* force re-painting the indicators */
+    FREE(con->deco_render_params);
+
+    tree_flatten(croot);
+
+    ipc_send_workspace_focus_event(ws, old_ws);
+}
+
 /*
  * Moves the current container in the given direction (D_LEFT, D_RIGHT,
  * D_UP, D_DOWN).
@@ -103,7 +146,8 @@ void tree_move(int direction) {
     }
 
     if (con->parent->type == CT_WORKSPACE && con_num_children(con->parent) == 1) {
-        DLOG("This is the only con on this workspace, not doing anything\n");
+        /* This is the only con on this workspace */
+        move_to_output_directed(con, direction);
         return;
     }
 
@@ -124,7 +168,7 @@ void tree_move(int direction) {
             if (con_inside_floating(con)) {
                 /* 'con' should be moved out of a floating container */
                 DLOG("Inside floating, moving to workspace\n");
-                attach_to_workspace(con, con_get_workspace(con));
+                attach_to_workspace(con, con_get_workspace(con), direction);
                 goto end;
             }
             DLOG("Force-changing orientation\n");
@@ -154,12 +198,15 @@ void tree_move(int direction) {
                 return;
             }
 
-            /* If there was no con with which we could swap the current one, search
-             * again, but starting one level higher. If we are on the workspace
-             * level, don’t do that. The result would be a force change of
-             * workspace orientation, which is not necessary. */
-            if (con->parent == con_get_workspace(con))
+            if (con->parent == con_get_workspace(con)) {
+                /*  If we couldn't find a place to move it on this workspace,
+                 *  try to move it to a workspace on a different output */
+                move_to_output_directed(con, direction);
                 return;
+            }
+
+            /* If there was no con with which we could swap the current one,
+             * search again, but starting one level higher. */
             same_orientation = con_parent_with_orientation(con->parent, o);
         }
     } while (same_orientation == NULL);
