@@ -1,8 +1,10 @@
+#undef I3__FILE__
+#define I3__FILE__ "output.c"
 /*
  * vim:ts=4:sw=4:expandtab
  *
  * i3 - an improved dynamic tiling window manager
- * © 2009-2011 Michael Stapelberg and contributors (see also: LICENSE)
+ * © 2009-2013 Michael Stapelberg and contributors (see also: LICENSE)
  *
  * output.c: Output (monitor) related functions.
  *
@@ -20,6 +22,5 @@ Con *output_get_content(Con *output) {
         if (child->type == CT_CON)
             return child;
 
-    ELOG("output_get_content() called on non-output %p\n", output);
-    assert(false);
+    return NULL;
 }
